@@ -1,4 +1,4 @@
-{-# LANGUAGE RecordWildCards, ScopedTypeVariables, DeriveDataTypeable, GeneralizedNewtypeDeriving #-}
+{-# LANGUAGE RecordWildCards, ScopedTypeVariables, DeriveDataTypeable, GeneralizedNewtypeDeriving, CPP #-}
 
 module Recipe.All(recipes) where
 
@@ -21,17 +21,19 @@ import Recipe.Cabal
 import Hoogle
 import qualified Paths_hoogle as V
 import Data.Version
-import CmdLine.All
+import CmdLine.All as C
 
 
 -- CmdLine is guaranteed to be a constructor of type Data
-recipes :: CmdLine -> IO ()
+recipes :: C.CmdLine -> IO ()
 recipes opt@Data{..} = withModeGlobalRead $ do
     hSetBuffering stdout NoBuffering
     createDirectoryIfMissing True datadir
     withDirectory datadir $ do
         when redownload $ do
-            forM_ (urls opt) $ \(file,_) -> removeFile_ $ "downloads" </> file
+            if nodownload
+              then error "Downloads are disabled, cannot re-download"
+              else forM_ (urls opt) $ \(file,_) -> removeFile_ $ "downloads" </> file
         when rebuild $ removeFile ".shake.database"
         (count, file) <- withWarnings $ \warn ->
             shake shakeOptions{shakeVersion=showVersion V.version, shakeThreads=threads, shakeProgress=progressSimple} $
@@ -43,19 +45,21 @@ recipes opt@Data{..} = withModeGlobalRead $ do
 newtype CabalVersion = CabalVersion String deriving (Show,Typeable,Eq,Hashable,Binary,NFData)
 newtype HoogleVersion = HoogleVersion String deriving (Show,Typeable,Eq,Hashable,Binary,NFData)
 
-rules :: CmdLine -> ([String] -> IO ()) -> Rules ()
+rules :: C.CmdLine -> ([String] -> IO ()) -> Rules ()
 rules opts@Data{..} warn = do
     let srcCabal name ver = "downloads/cabal" </> name </> ver </> name <.> "cabal"
     let srcHoogle name ver = "downloads/hoogle" </> name </> ver </> "doc" </> "html" </> name <.> "txt"
 
     (\x -> "downloads/*" ?== x &&
            isJust (lookup (takeFileName x) (urls opts))) ?> \out -> do
+        when nodownload $
+             error "Downloads are disabled; you need to acquire the source files manually."
         let Just url = lookup (takeFileName out) (urls opts)
         putNormal $ "Downloading " ++ out
-        wget url out
+        wget opts url out
         putNormal $ "Downloaded " ++ out
 
-    "downloads/*.cache" *> \out -> do
+    "downloads/*.cache" %> \out -> do
         let src = dropExtension out
         need [src]
         src <- liftIO $ readFileUtf8' src
@@ -64,12 +68,12 @@ rules opts@Data{..} warn = do
             old <- readFileUtf8' out
             when (src /= old) $ writeFileUtf8 out src
 
-    "//*.tar" *> \out -> do
+    "//*.tar" %> \out -> do
         let src = out <.> "gz"
         need [src]
         ungzip src out
 
-    "//*.index" *> \out -> do
+    "//*.index" %> \out -> do
         let src = out -<.> "tar"
         need [src]
         putNormal $ "Extracting tar file " ++ out
@@ -92,20 +96,20 @@ rules opts@Data{..} warn = do
         need $ map (<.> "hoo") $ ["all" | "all" `elem` bad] ++ good
 
     alternatives $ do -- Match *.txt
-        "keyword.txt" *> \out -> do
+        "keyword.txt" %> \out -> do
             let src = "downloads/keyword.htm.cache"
             need [src]
             contents <- liftIO $ readFileUtf8' src
             liftIO $ writeFileUtf8 out $ translateKeywords contents
 
-        "default.txt" *> \out -> do
+        "default.txt" %> \out -> do
             writeFileLines out ["@combine keyword","@combine package","@combine platform"]
 
-        "platform.txt" *> \out -> do
+        "platform.txt" %> \out -> do
             contents <- readFile' "downloads/platform.cabal.cache"
             writeFileLines out ["@combine " ++ x | x <- platformPackages contents]
 
-        "package.txt" *> \out -> do
+        "package.txt" %> \out -> do
             cabs <- index "downloads/cabal.index"
             xs <- liftIO $ forM (Map.toList cabs) $ \(name,ver) -> do
                 src <- try $ readCabal $ srcCabal name ver
@@ -116,7 +120,7 @@ rules opts@Data{..} warn = do
                         ["--","-- Version " ++ ver, "@url package/" ++ name, "@entry package " ++ name]
             liftIO $ writeFileUtf8 out $ unlines $ ("@url " ++ hackage) : "@package package" : concat xs
 
-        "*.txt" *> \out -> do
+        "*.txt" %> \out -> do
             let name = takeBaseName out
                 base = name == "base"
             cab <- fmap (fmap $ srcCabal name) $ verCabal (CabalVersion name)
@@ -155,7 +159,7 @@ rules opts@Data{..} warn = do
                     i <- imported $ t <.> "txt"
                     fmap (i++) $ genImported (Set.insert t seen) (fst (splitDeps i) ++ odo)
 
-        "*.hoo" *> \out -> do
+        "*.hoo" %> \out -> do
             let src = out -<.> "txt"
             need [src]
             contents <- liftIO $ fmap lines $ readFileUtf8' src
@@ -174,9 +178,9 @@ rules opts@Data{..} warn = do
                 liftIO $ warn [takeBaseName out ++ ": " ++ show e | e <- err]
 
 
-urls :: CmdLine -> [(FilePath, URL)]
+urls :: C.CmdLine -> [(FilePath, URL)]
 urls Data{..} = let (*) = (,) in
-    ["keyword.htm" * "http://www.haskell.org/haskellwiki/Keywords"
+    ["keyword.htm" * "http://wiki.haskell.org/Keywords"
     ,"platform.cabal" * "http://code.galois.com/darcs/haskell-platform/haskell-platform.cabal"
     ,"base.txt" * "http://www.haskell.org/hoogle/base.txt"
     ,"cabal.tar.gz" * (hackage ++ "packages/index.tar.gz")
